import pytest

from zino.oid import OID


class TestOIDIsPrefix:
    def test_return_true_if_prefix(self):
        oid = OID(".1.2.3.4.5")
        assert oid.is_a_prefix_of(".1.2.3.4.5.6")

    def test_return_false_if_not_prefix(self):
        oid = OID(".5.4.3.2.1")
        assert not oid.is_a_prefix_of(".1.2.3.4.5.6")

    def test_return_false_if_prefix_equal_to_oid(self):
        oid = OID(".5.4.3.2.1")
        assert not oid.is_a_prefix_of(".5.4.3.2.1")


class TestOIDStripPrefix:
    def test_valid_prefix_is_stripped(self):
        oid = ".1.2.3.4.5"
        prefix = ".1.2"
        stripped_oid = OID(oid).strip_prefix(prefix)
        assert str(stripped_oid) == ".3.4.5"

    def test_invalid_prefix_is_not_stripped(self):
        oid = ".1.2.3.4.5"
        prefix = ".5.4"
        stripped_oid = OID(oid).strip_prefix(prefix)
        assert str(stripped_oid) == oid


def test_can_add_nodes_to_oid():
    added_oid = OID(".1.2.3.4.5") + "6.7.8"
    assert str(added_oid) == ".1.2.3.4.5.6.7.8"


def test_can_create_oid_from_bytestring():
    oid_bytestring = b".1.2.3.4"
    oid = OID(oid_bytestring)
    assert str(oid) == ".1.2.3.4"


def test_when_oid_contains_non_numeric_elements_it_should_raise_typeerror():
    with pytest.raises(TypeError):
        OID((1, 2, 3, "foo", 4))



import asyncio
import ipaddress
import os
from datetime import timedelta
from shutil import which

import pytest
import pytest_asyncio
from retry import retry

from zino.planned_maintenance import PlannedMaintenances
from zino.state import ZinoState
from zino.statemodels import (
    DeviceMaintenance,
    DeviceState,
    InterfaceState,
    MatchType,
    Port,
    PortStateMaintenance,
)
from zino.time import now
from zino.trapd import netsnmpy_backend, pysnmp_backend


def pytest_configure(config):
    import os

    # Load the default SNMP back-end for the test suite.  Tests for specific back-end will import directly from those.
    from zino.snmp import import_snmp_backend
    from zino.trapd import import_trap_backend

    import_snmp_backend()
    import_trap_backend()

    from netsnmpy import netsnmp

    from zino.snmp import get_vendored_mib_directory

    # Ensure that the vendored MIBs are loaded
    os.environ["MIBS"] = "ALL"
    vendored_mibs = get_vendored_mib_directory()
    print(f"Setting MIBDIRS to {vendored_mibs}")
    os.environ["MIBDIRS"] = f"{vendored_mibs}"
    netsnmp.load_mibs()
    modules = ", ".join(sorted(netsnmp.get_loaded_mibs()))
    print(f"Loaded MIB modules: {modules}")

    # Every test should operate with a fresh SNMP session object:
    # Disable re-use of SNMP sessions for testing purposes
    from zino import snmp

    snmp._snmp_sessions = None


@pytest.fixture
def polldevs_conf(tmp_path):
    name = tmp_path.joinpath("polldevs.cf")
    with open(name, "w") as conf:
        conf.write(
            """# polldevs test config
            default interval: 5
            default community: foobar
            default domain: uninett.no
            default statistics: yes
            default snmpversion: v2c

            name: example-gw
            address: 10.0.42.1

            name: example-gw2
            address: 10.0.43.1"""  # Lack of a new-line here is intentional to test the parser
        )
    yield name


@pytest.fixture
def polldevs_conf_with_single_router(tmp_path):
    name = tmp_path / "polldevs-single.cf"
    with open(name, "w") as conf:
        conf.write(
            """# polldevs test config
            default interval: 5
            default community: foobar
            default domain: uninett.no
            default statistics: yes
            default snmpversion: v2c

            name: example-gw
            address: 10.0.42.1
            """
        )
    yield name


@pytest.fixture
def polldevs_conf_with_no_routers(tmp_path):
    name = tmp_path / "polldevs-empty.cf"
    with open(name, "w") as conf:
        conf.write(
            """# polldevs test config
            default interval: 5
            default community: foobar
            default domain: uninett.no
            default statistics: yes
            default snmpversion: v2c
            """
        )
    yield name


@pytest.fixture
def invalid_polldevs_conf(tmp_path):
    name = tmp_path.joinpath("polldevs.cf")
    with open(name, "w") as conf:
        conf.write(
            """# polldevs test config
            lalala
            """
        )
    yield name


@pytest.fixture
def secrets_file(tmp_path):
    name = tmp_path / "secrets"
    with open(name, "w") as conf:
        conf.write("""user1 password123""")
    yield name


@pytest.fixture
def zino_conf(tmp_path, polldevs_conf_with_no_routers, secrets_file):
    name = tmp_path / "zino.toml"
    with open(name, "w") as conf:
        conf.write(
            f"""
            [authentication]
            file = "{secrets_file}"
            [polling]
            file = "{polldevs_conf_with_no_routers}"
            """
        )
    yield name


@pytest.fixture
def zino_conf_with_non_existent_pollfile(tmp_path, secrets_file):
    name = tmp_path / "zino-no-pollfile.toml"
    with open(name, "w") as conf:
        conf.write(
            f"""
            [authentication]
            file = "{secrets_file}"
            [polling]
            file = "{tmp_path}/non-existent-pollfile.cf"
            """
        )
    yield name


@pytest.fixture
def invalid_zino_conf(tmp_path):
    name = tmp_path / "invalid-zino.toml"
    with open(name, "w") as conf:
        conf.write(
            """
                [archiving]
                old_events_dir = abc
            """
        )
    yield name


@pytest.fixture
def invalid_values_zino_conf(tmp_path):
    name = tmp_path / "invalid-config-values.toml"
    with open(name, "w") as conf:
        conf.write(
            """
                [archiving]
                old_events_dir = false
            """
        )
    yield name


@pytest.fixture
def extra_keys_zino_conf(tmp_path):
    name = tmp_path / "extra-keys.toml"
    with open(name, "w") as conf:
        conf.write(
            """
                [archiving]
                typo = "old-zino-events"
            """
        )
    yield name


@pytest.fixture(scope="session")
def event_loop():
    """Redefine pytest-asyncio's event_loop fixture to have a session scope"""
    policy = asyncio.get_event_loop_policy()
    loop = policy.new_event_loop()
    yield loop
    loop.close()


@pytest_asyncio.fixture(scope="session")
async def snmpsim(snmpsimd_path, snmp_fixture_directory, snmp_test_port):
    """Sets up an external snmpsimd process so that SNMP communication can be simulated
    by the test that declares a dependency to this fixture. Data fixtures are loaded
    from the snmp_fixtures subdirectory.
    """
    arguments = [
        f"--data-dir={snmp_fixture_directory}",
        "--log-level=error",
        f"--agent-udpv4-endpoint=127.0.0.1:{snmp_test_port}",
    ]
    print(f"Running {snmpsimd_path} with args: {arguments!r}")
    proc = await asyncio.create_subprocess_exec(snmpsimd_path, *arguments)

    @retry(Exception, tries=3, delay=0.5, backoff=2)
    async def _wait_for_snmpsimd():
        if await _verify_localhost_snmp_response(snmp_test_port):
            return True
        else:
            raise TimeoutError("Still waiting for snmpsimd to listen for queries")

    await _wait_for_snmpsimd()

    yield
    proc.kill()


@pytest.fixture(scope="session")
def snmpsimd_path():
    snmpsimd = which("snmpsim-command-responder")
    assert snmpsimd, "Could not find snmpsim-command-responder"
    yield snmpsimd


@pytest.fixture(scope="session")
def snmp_fixture_directory():
    this_directory = os.path.dirname(__file__)
    fixture_dir = os.path.join(this_directory, "snmp_fixtures")
    assert os.path.isdir(fixture_dir)
    yield fixture_dir


@pytest.fixture(scope="session")
def snmp_test_port():
    yield 1024


@pytest.fixture
def state_with_localhost():
    localhost = ipaddress.ip_address("127.0.0.1")
    state = ZinoState()
    state.devices.devices["localhost"] = DeviceState(name="localhost", addresses={localhost})
    state.addresses[localhost] = "localhost"
    yield state


@pytest_asyncio.fixture
async def localhost_pysnmp_receiver(state_with_localhost, unused_udp_port, event_loop) -> pysnmp_backend.TrapReceiver:
    """Yields a TrapReceiver instance with a standardized setup for running external tests on localhost"""
    receiver = pysnmp_backend.TrapReceiver(
        address="127.0.0.1", port=unused_udp_port, loop=event_loop, state=state_with_localhost
    )
    receiver.add_community("public")
    await receiver.open()
    yield receiver
    receiver.close()


@pytest_asyncio.fixture
async def localhost_netsnmpy_receiver(
    state_with_localhost, unused_udp_port, event_loop
) -> netsnmpy_backend.TrapReceiver:
    """Yields a TrapReceiver instance with a standardized setup for running external tests on localhost"""
    receiver = netsnmpy_backend.TrapReceiver(
        address="127.0.0.1", port=unused_udp_port, loop=event_loop, state=state_with_localhost
    )
    receiver.add_community("public")
    await receiver.open()
    yield receiver
    receiver.close()


async def _verify_localhost_snmp_response(port: int):
    """Verifies that the snmpsimd fixture process is responding, by using PySNMP directly to query it."""

    from pysnmp.hlapi.asyncio import (
        CommunityData,
        ContextData,
        ObjectIdentity,
        ObjectType,
        SnmpEngine,
        UdpTransportTarget,
        nextCmd,
    )

    responses = await nextCmd(
        SnmpEngine(),
        CommunityData("public"),
        UdpTransportTarget(("localhost", port)),
        ContextData(),
        ObjectType(ObjectIdentity("SNMPv2-MIB", "sysObjectID")),
    )
    return responses


@pytest.fixture
def state_with_localhost_with_port(state_with_localhost):
    port = Port(ifindex=1, ifdescr="eth0", ifalias="eth0alias", state=InterfaceState.UP)
    device = state_with_localhost.devices.devices["localhost"]
    device.boot_time = now() - timedelta(minutes=10)
    device.ports[port.ifindex] = port
    yield state_with_localhost


@pytest.fixture
def pms():
    return PlannedMaintenances()


@pytest.fixture
def active_device_pm(pms):
    return pms.create_planned_maintenance(
        start_time=now() - timedelta(days=1),
        end_time=now() + timedelta(days=1),
        pm_class=DeviceMaintenance,
        match_type=MatchType.EXACT,
        match_expression="device",
        match_device="device",
    )


@pytest.fixture
def active_portstate_pm(pms):
    return pms.create_planned_maintenance(
        start_time=now() - timedelta(days=1),
        end_time=now() + timedelta(days=1),
        pm_class=PortStateMaintenance,
        match_type=MatchType.REGEXP,
        match_expression="port",
        match_device="device",
    )


@pytest.fixture
def ended_pm(pms):
    return pms.create_planned_maintenance(
        start_time=now() - timedelta(days=1),
        end_time=now() - timedelta(minutes=10),
        pm_class=DeviceMaintenance,
        match_type=MatchType.EXACT,
        match_expression="device",
        match_device="device",
    )



import getpass
import grp
import hashlib
import logging
import os
import pwd
import secrets
import subprocess
import time
from argparse import Namespace
from datetime import timedelta
from unittest.mock import Mock, patch

import pexpect
import pytest

from zino import zino
from zino.scheduler import get_scheduler
from zino.time import now


def test_zino_version_should_be_available():
    from zino import version

    assert version.__version__
    assert version.__version_tuple__


def test_zino_help_screen_should_not_crash():
    assert subprocess.check_call(["zino", "--help"]) == 0


def test_zino_should_not_crash_right_away(polldevs_conf_with_no_routers, unused_udp_port, zino_conf):
    """This tests that the main function runs Zino for at least 2 seconds"""
    seconds_to_run_for = 2
    subprocess.check_call(
        [
            "zino",
            "--stop-in",
            str(seconds_to_run_for),
            "--polldevs",
            str(polldevs_conf_with_no_routers),
            "--config-file",
            str(zino_conf),
            "--trap-port",
            str(unused_udp_port),
        ]
    )


def test_zino_should_run_with_pollfile_name_in_config_file(polldevs_conf_with_no_routers, unused_udp_port, zino_conf):
    """This tests that the main function runs Zino for at least 2 seconds when
    the name of the pollfile is defined in the config file
    """
    seconds_to_run_for = 2
    subprocess.check_call(
        [
            "zino",
            "--stop-in",
            str(seconds_to_run_for),
            "--config-file",
            str(zino_conf),
            "--trap-port",
            str(unused_udp_port),
        ]
    )


def test_zino_should_not_run_without_pollfile(zino_conf_with_non_existent_pollfile, unused_udp_port):
    """This tests that the main function does not Zino for at least 2 seconds when
    the name of the pollfile is defined in the config file, but does not exist
    """
    with pytest.raises(subprocess.CalledProcessError):
        seconds_to_run_for = 2
        subprocess.check_call(
            [
                "zino",
                "--stop-in",
                str(seconds_to_run_for),
                "--config-file",
                str(zino_conf_with_non_existent_pollfile),
                "--trap-port",
                str(unused_udp_port),
            ]
        )


def test_when_args_specified_config_file_does_not_exist_then_load_config_should_exit(tmp_path):
    args = Mock(config_file=tmp_path / "non_existent_file.toml")
    with pytest.raises(SystemExit):
        zino.load_config(args)


def test_when_logging_config_is_invalid_then_apply_logging_config_should_exit():
    with pytest.raises(SystemExit):
        zino.apply_logging_config({"loggers": {"zino": {"level": "invalid"}}})


def test_zino_should_not_run_with_invalid_conf_file(invalid_zino_conf, unused_udp_port):
    """This tests that the main function does not Zino for at least 2 seconds when
    the name of the pollfile is defined in the config file, but does not exist
    """
    with pytest.raises(subprocess.CalledProcessError):
        seconds_to_run_for = 2
        subprocess.check_call(
            [
                "zino",
                "--stop-in",
                str(seconds_to_run_for),
                "--config-file",
                str(invalid_zino_conf),
                "--trap-port",
                str(unused_udp_port),
            ]
        )


def test_zino_argparser_works(polldevs_conf):
    parser = zino.parse_args(["--polldevs", str(polldevs_conf)])
    assert isinstance(parser, Namespace)


@pytest.mark.skipif(os.geteuid() == 0, reason="cannot test while being root")
def test_when_unprivileged_user_asks_for_privileged_port_zino_should_exit_with_error(polldevs_conf_with_no_routers):
    seconds_to_run_for = 5
    with pytest.raises(subprocess.CalledProcessError):
        subprocess.check_call(
            [
                "zino",
                "--stop-in",
                str(seconds_to_run_for),
                "--polldevs",
                str(polldevs_conf_with_no_routers),
                "--trap-port",
                "162",
            ]
        )


@patch("zino.zino.switch_to_user")
async def test_when_args_specify_user_zino_init_event_loop_should_attempt_to_switch_users(switch_to_user, event_loop):
    """Detect attempt to user switching by patching in a mock exception.  This is to avoid setting up the full Zino
    daemon and mucking up the event loop and state, by ensuring we exit as soon as switch_to_user is called.
    """

    class MockError(Exception):
        pass

    zino.switch_to_user.side_effect = MockError()
    with pytest.raises(MockError):
        try:
            zino.init_event_loop(args=Mock(trap_port=0, user="nobody"), loop=event_loop)
        except MockError:
            raise
        except Exception:
            pass


class TestZinoRescheduleDumpStateOnCommit:
    def test_when_more_than_10_seconds_remains_until_next_dump_it_should_reschedule(self):
        scheduler = get_scheduler()
        mock_job = Mock(next_run_time=now() + timedelta(minutes=5))
        mock_event = Mock(id=42)

        with patch.object(scheduler, "get_job") as get_job:
            get_job.return_value = mock_job
            zino.reschedule_dump_state_on_commit(mock_event)
            assert mock_job.modify.called

    def test_when_less_than_10_seconds_remains_until_next_dump_it_should_not_reschedule(self):
        scheduler = get_scheduler()
        mock_job = Mock(next_run_time=now() + timedelta(seconds=5))
        mock_event = Mock(id=42)

        with patch.object(scheduler, "get_job") as get_job:
            get_job.return_value = mock_job
            zino.reschedule_dump_state_on_commit(mock_event)
            assert not mock_job.modify.called


class TestZinoRescheduleDumpStateOnPmChange:
    def test_when_more_than_10_seconds_remain_until_next_dump_it_should_reschedule(self):
        scheduler = get_scheduler()
        mock_job = Mock(next_run_time=now() + timedelta(minutes=5))

        with patch.object(scheduler, "get_job") as get_job:
            get_job.return_value = mock_job
            zino.reschedule_dump_state_on_pm_change()
            assert mock_job.modify.called

    def test_when_less_than_10_seconds_remain_until_next_dump_it_should_not_reschedule(self):
        scheduler = get_scheduler()
        mock_job = Mock(next_run_time=now() + timedelta(seconds=5))

        with patch.object(scheduler, "get_job") as get_job:
            get_job.return_value = mock_job
            zino.reschedule_dump_state_on_pm_change()
            assert not mock_job.modify.called


class TestSwitchUser:
    def test_when_switching_to_current_user_it_should_succeed(self):
        username = getpass.getuser()
        assert zino.switch_to_user(username)

    def test_when_user_does_not_exist_it_should_return_false(self):
        random_username = secrets.token_hex(5)
        print(f"attempting switch to user {random_username!r}")
        assert not zino.switch_to_user(random_username)

    @patch("pwd.getpwnam")
    @patch("grp.getgrall")
    @patch("os.setgid")
    @patch("os.setgroups")
    @patch("os.setuid")
    def test_when_user_exists_it_should_switch_correctly_to_uid_gid_and_groups(
        self, getpwnam, getgrall, setgid, setgroups, setuid
    ):
        random_username = secrets.token_hex(5)
        pwd.getpwnam.return_value = Mock(pw_uid=666, pw_gid=999)
        grp.getgrall.return_value = [Mock(gr_gid=4242, gr_mem=[random_username])]

        assert zino.switch_to_user(random_username)
        os.setgid.assert_called_once_with(999)
        os.setuid.assert_called_once_with(666)
        os.setgroups.assert_called_once_with([4242])

    @patch("pwd.getpwnam")
    @patch("grp.getgrall")
    @patch("os.setgid")
    @patch("os.setgroups")
    @patch("os.setuid")
    def test_when_setgid_raises_oserror_it_should_return_false(self, getpwnam, getgrall, setgid, setgroups, setuid):
        random_username = secrets.token_hex(5)
        pwd.getpwnam.return_value = Mock(pw_uid=666, pw_gid=999)
        grp.getgrall.return_value = [Mock(gr_gid=4242, gr_mem=[random_username])]
        os.setgid.side_effect = OSError("Mock error")

        assert not zino.switch_to_user(random_username)


class TestCountReachableObjects:
    def test_it_should_count_queried_classes(self):
        counts = zino._count_reachable_objects(str, int)
        assert int in counts
        assert counts[int] > 0
        assert str in counts
        assert counts[str] > 0

    def test_it_should_not_count_non_queried_classes(self):
        counts = zino._count_reachable_objects(str)
        assert int not in counts


class TestLogSnmpSessionState:
    def test_when_netsnmpy_is_backend_it_should_log_low_level_details(self, state_with_localhost, caplog):
        import zino.snmp.netsnmpy_backend as backend

        with patch.object(zino.state, attribute="state", new=state_with_localhost):
            with patch.object(zino, attribute="import_snmp_backend") as import_snmp_backend:
                import_snmp_backend.return_value = backend
                with caplog.at_level(logging.DEBUG):
                    zino.log_snmp_session_stats()
                    assert "gc reachable (low-level)=" in caplog.text

    def test_when_pysnmp_is_backend_it_should_not_log_low_level_details(self, state_with_localhost, caplog):
        import zino.snmp.pysnmp_backend as backend

        with patch.object(zino.state, attribute="state", new=state_with_localhost):
            with patch.object(zino, attribute="import_snmp_backend") as import_snmp_backend:
                import_snmp_backend.return_value = backend
                with caplog.at_level(logging.DEBUG):
                    zino.log_snmp_session_stats()
                    assert "gc reachable (low-level)=" not in caplog.text

    def test_when_debug_logging_is_not_enabled_it_should_not_log_anything(self, state_with_localhost, caplog):
        import zino.snmp.netsnmpy_backend as backend

        with patch.object(zino.state, attribute="state", new=state_with_localhost):
            with patch.object(zino, attribute="import_snmp_backend") as import_snmp_backend:
                import_snmp_backend.return_value = backend
                with caplog.at_level(logging.INFO):
                    zino.log_snmp_session_stats()
                    assert "gc reachable" not in caplog.text


class TestSecretsFileConfiguration:
    def test_users_from_secrets_file_should_be_able_to_authenticate(
        self, polldevs_conf_with_no_routers, zino_conf, secrets_file
    ):
        """Test connecting to the server, handling SHA1 challenge, and asserting successful login."""
        # Start the process
        zino_process = subprocess.Popen(["zino", "--config-file", str(zino_conf), "--trap-port", "0"])
        with open(secrets_file, "r") as f:
            username, password = f.readline().strip().split(" ", 1)
        time.sleep(1)  # Wait for the server to start
        try:
            client = pexpect.spawn("telnet localhost 8001", timeout=5)
            client.expect("200 ([a-f0-9]+) Hello, there")
            challenge = client.match.group(1).decode("utf-8")

            response = hashlib.sha1((challenge + " " + password).encode("utf-8")).hexdigest()

            client.sendline(f"USER {username} {response}")
            client.expect(["200 ok", "500 Authentication failure"])
            assert "200 ok" in client.after.decode("utf-8")

        finally:
            zino_process.terminate()



#
# PySNMP MIB module OLD-CISCO-TS-MIB (http://snmplabs.com/pysmi)
# ASN.1 source file:///home/mvold/mibs/cisco/OLD-CISCO-TS-MIB.my
# Produced by pysmi-0.3.4 at Thu Jul 18 11:03:56 2024
# On host agrajag platform Linux version 6.6.37 by user mvold
# Using Python version 3.11.9 (main, Apr  2 2024, 08:25:04) [GCC 13.2.0]
#
Integer, ObjectIdentifier, OctetString = mibBuilder.importSymbols("ASN1", "Integer", "ObjectIdentifier", "OctetString")
NamedValues, = mibBuilder.importSymbols("ASN1-ENUMERATION", "NamedValues")
SingleValueConstraint, ValueSizeConstraint, ValueRangeConstraint, ConstraintsUnion, ConstraintsIntersection = mibBuilder.importSymbols("ASN1-REFINEMENT", "SingleValueConstraint", "ValueSizeConstraint", "ValueRangeConstraint", "ConstraintsUnion", "ConstraintsIntersection")
local, = mibBuilder.importSymbols("CISCO-SMI", "local")
NotificationGroup, ModuleCompliance = mibBuilder.importSymbols("SNMPv2-CONF", "NotificationGroup", "ModuleCompliance")
Bits, Counter64, IpAddress, iso, Gauge32, Integer32, ModuleIdentity, Unsigned32, MibIdentifier, ObjectIdentity, MibScalar, MibTable, MibTableRow, MibTableColumn, NotificationType, Counter32, TimeTicks = mibBuilder.importSymbols("SNMPv2-SMI", "Bits", "Counter64", "IpAddress", "iso", "Gauge32", "Integer32", "ModuleIdentity", "Unsigned32", "MibIdentifier", "ObjectIdentity", "MibScalar", "MibTable", "MibTableRow", "MibTableColumn", "NotificationType", "Counter32", "TimeTicks")
TextualConvention, DisplayString = mibBuilder.importSymbols("SNMPv2-TC", "TextualConvention", "DisplayString")
lts = MibIdentifier((1, 3, 6, 1, 4, 1, 9, 2, 9))
tsLines = MibScalar((1, 3, 6, 1, 4, 1, 9, 2, 9, 1), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLines.setStatus('mandatory')
ltsLineTable = MibTable((1, 3, 6, 1, 4, 1, 9, 2, 9, 2), )
if mibBuilder.loadTexts: ltsLineTable.setStatus('mandatory')
ltsLineEntry = MibTableRow((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1), ).setIndexNames((0, "OLD-CISCO-TS-MIB", "tsLineNumber"))
if mibBuilder.loadTexts: ltsLineEntry.setStatus('mandatory')
tsLineActive = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 1), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineActive.setStatus('mandatory')
tsLineType = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 2), Integer32().subtype(subtypeSpec=ConstraintsUnion(SingleValueConstraint(1, 2, 3, 4, 5, 6))).clone(namedValues=NamedValues(("unknown", 1), ("console", 2), ("terminal", 3), ("line-printer", 4), ("virtual-terminal", 5), ("auxiliary", 6)))).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineType.setStatus('mandatory')
tsLineAutobaud = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 3), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineAutobaud.setStatus('mandatory')
tsLineSpeedin = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 4), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineSpeedin.setStatus('mandatory')
tsLineSpeedout = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 5), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineSpeedout.setStatus('mandatory')
tsLineFlow = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 6), Integer32().subtype(subtypeSpec=ConstraintsUnion(SingleValueConstraint(1, 2, 3, 4, 5, 6, 7, 8))).clone(namedValues=NamedValues(("unknown", 1), ("none", 2), ("software-input", 3), ("software-output", 4), ("software-both", 5), ("hardware-input", 6), ("hardware-output", 7), ("hardware-both", 8)))).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineFlow.setStatus('mandatory')
tsLineModem = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 7), Integer32().subtype(subtypeSpec=ConstraintsUnion(SingleValueConstraint(1, 2, 3, 4, 5, 6, 7))).clone(namedValues=NamedValues(("unknown", 1), ("none", 2), ("call-in", 3), ("call-out", 4), ("cts-required", 5), ("ri-is-cd", 6), ("inout", 7)))).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineModem.setStatus('mandatory')
tsLineLoc = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 8), DisplayString()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineLoc.setStatus('mandatory')
tsLineTerm = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 9), DisplayString()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineTerm.setStatus('mandatory')
tsLineScrlen = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 10), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineScrlen.setStatus('mandatory')
tsLineScrwid = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 11), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineScrwid.setStatus('mandatory')
tsLineEsc = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 12), DisplayString()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineEsc.setStatus('mandatory')
tsLineTmo = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 13), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineTmo.setStatus('mandatory')
tsLineSestmo = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 14), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineSestmo.setStatus('mandatory')
tsLineRotary = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 15), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineRotary.setStatus('mandatory')
tsLineUses = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 16), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineUses.setStatus('mandatory')
tsLineNses = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 17), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineNses.setStatus('mandatory')
tsLineUser = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 18), DisplayString()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineUser.setStatus('mandatory')
tsLineNoise = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 19), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineNoise.setStatus('mandatory')
tsLineNumber = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 20), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineNumber.setStatus('mandatory')
tsLineTimeActive = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 2, 1, 21), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tsLineTimeActive.setStatus('mandatory')
ltsLineSessionTable = MibTable((1, 3, 6, 1, 4, 1, 9, 2, 9, 3), )
if mibBuilder.loadTexts: ltsLineSessionTable.setStatus('mandatory')
ltsLineSessionEntry = MibTableRow((1, 3, 6, 1, 4, 1, 9, 2, 9, 3, 1), ).setIndexNames((0, "OLD-CISCO-TS-MIB", "tslineSesLine"), (0, "OLD-CISCO-TS-MIB", "tslineSesSession"))
if mibBuilder.loadTexts: ltsLineSessionEntry.setStatus('mandatory')
tslineSesType = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 3, 1, 1), Integer32().subtype(subtypeSpec=ConstraintsUnion(SingleValueConstraint(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11))).clone(namedValues=NamedValues(("unknown", 1), ("pad", 2), ("stream", 3), ("rlogin", 4), ("telnet", 5), ("tcp", 6), ("lat", 7), ("mop", 8), ("slip", 9), ("xremote", 10), ("rshell", 11)))).setMaxAccess("readonly")
if mibBuilder.loadTexts: tslineSesType.setStatus('mandatory')
tslineSesDir = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 3, 1, 2), Integer32().subtype(subtypeSpec=ConstraintsUnion(SingleValueConstraint(1, 2, 3))).clone(namedValues=NamedValues(("unknown", 1), ("incoming", 2), ("outgoing", 3)))).setMaxAccess("readonly")
if mibBuilder.loadTexts: tslineSesDir.setStatus('mandatory')
tslineSesAddr = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 3, 1, 3), IpAddress()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tslineSesAddr.setStatus('mandatory')
tslineSesName = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 3, 1, 4), DisplayString()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tslineSesName.setStatus('mandatory')
tslineSesCur = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 3, 1, 5), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tslineSesCur.setStatus('mandatory')
tslineSesIdle = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 3, 1, 6), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tslineSesIdle.setStatus('mandatory')
tslineSesLine = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 3, 1, 7), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tslineSesLine.setStatus('mandatory')
tslineSesSession = MibTableColumn((1, 3, 6, 1, 4, 1, 9, 2, 9, 3, 1, 8), Integer32()).setMaxAccess("readonly")
if mibBuilder.loadTexts: tslineSesSession.setStatus('mandatory')
tsMsgTtyLine = MibScalar((1, 3, 6, 1, 4, 1, 9, 2, 9, 4), Integer32()).setMaxAccess("readwrite")
if mibBuilder.loadTexts: tsMsgTtyLine.setStatus('mandatory')
tsMsgIntervaltim = MibScalar((1, 3, 6, 1, 4, 1, 9, 2, 9, 5), Integer32()).setMaxAccess("readwrite")
if mibBuilder.loadTexts: tsMsgIntervaltim.setStatus('mandatory')
tsMsgDuration = MibScalar((1, 3, 6, 1, 4, 1, 9, 2, 9, 6), Integer32()).setMaxAccess("readwrite")
if mibBuilder.loadTexts: tsMsgDuration.setStatus('mandatory')
tsMsgText = MibScalar((1, 3, 6, 1, 4, 1, 9, 2, 9, 7), DisplayString()).setMaxAccess("readwrite")
if mibBuilder.loadTexts: tsMsgText.setStatus('mandatory')
tsMsgTmpBanner = MibScalar((1, 3, 6, 1, 4, 1, 9, 2, 9, 8), Integer32().subtype(subtypeSpec=ConstraintsUnion(SingleValueConstraint(1, 2))).clone(namedValues=NamedValues(("no", 1), ("additive", 2)))).setMaxAccess("readwrite")
if mibBuilder.loadTexts: tsMsgTmpBanner.setStatus('mandatory')
tsMsgSend = MibScalar((1, 3, 6, 1, 4, 1, 9, 2, 9, 9), Integer32().subtype(subtypeSpec=ConstraintsUnion(SingleValueConstraint(1, 2, 3, 4))).clone(namedValues=NamedValues(("nothing", 1), ("reload", 2), ("messagedone", 3), ("abort", 4)))).setMaxAccess("readwrite")
if mibBuilder.loadTexts: tsMsgSend.setStatus('mandatory')
tsClrTtyLine = MibScalar((1, 3, 6, 1, 4, 1, 9, 2, 9, 10), Integer32()).setMaxAccess("readwrite")
if mibBuilder.loadTexts: tsClrTtyLine.setStatus('mandatory')
mibBuilder.exportSymbols("OLD-CISCO-TS-MIB", tsLines=tsLines, ltsLineSessionEntry=ltsLineSessionEntry, tsLineNumber=tsLineNumber, tsLineFlow=tsLineFlow, tsLineNoise=tsLineNoise, tsClrTtyLine=tsClrTtyLine, tsLineTimeActive=tsLineTimeActive, tslineSesType=tslineSesType, tsLineLoc=tsLineLoc, ltsLineEntry=ltsLineEntry, tsLineActive=tsLineActive, tsLineNses=tsLineNses, tslineSesSession=tslineSesSession, ltsLineTable=ltsLineTable, tsLineRotary=tsLineRotary, tsLineScrwid=tsLineScrwid, tsLineTerm=tsLineTerm, tsLineUses=tsLineUses, tsLineAutobaud=tsLineAutobaud, tsLineEsc=tsLineEsc, tsLineModem=tsLineModem, ltsLineSessionTable=ltsLineSessionTable, tsLineTmo=tsLineTmo, tslineSesName=tslineSesName, tsMsgTtyLine=tsMsgTtyLine, tslineSesDir=tslineSesDir, tsLineType=tsLineType, tslineSesIdle=tslineSesIdle, tslineSesLine=tslineSesLine, tsMsgSend=tsMsgSend, tsMsgTmpBanner=tsMsgTmpBanner, tsLineScrlen=tsLineScrlen, tsLineSpeedout=tsLineSpeedout, tsMsgText=tsMsgText, lts=lts, tsLineSpeedin=tsLineSpeedin, tsLineSestmo=tsLineSestmo, tslineSesCur=tslineSesCur, tsMsgDuration=tsMsgDuration, tsLineUser=tsLineUser, tsMsgIntervaltim=tsMsgIntervaltim, tslineSesAddr=tslineSesAddr)



"""This module implements a trap observer to ignore spammy traps"""

import logging
from typing import Optional, Set

from zino.trapd.base import TrapMessage, TrapObserver, TrapType

_logger = logging.getLogger(__name__)


class IgnoreTraps(TrapObserver):
    """Completely ignores spammy traps observed in Uninett, that we don't care about"""

    WANTED_TRAPS: Set[TrapType] = {
        ("BGP4-MIB", "bgpBackwardTransition"),
        ("BGP4-MIB", "bgpBackwardTransNotification"),
        ("BGP4-V2-MIB-JUNIPER", "jnxBgpM2BackwardTransition"),
        ("SNMPv2-MIB", "authenticationFailure"),
        ("CISCOTRAP-MIB", "tcpConnectionClose"),
    }

    async def handle_trap(self, trap: TrapMessage) -> Optional[bool]:
        return False  # Stop processing here!
